/*
 * govirt.h: main header
 *
 * Copyright (C) 2012 Red Hat, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library. If not, see
 * <http://www.gnu.org/licenses/>.
 *
 * Author: Christophe Fergeau <cfergeau@redhat.com>
 */
#ifndef __OVIRT_H__
#define __OVIRT_H__

#include <govirt/ovirt-enum-types.h>
#include <govirt/ovirt-api.h>
#include <govirt/ovirt-cdrom.h>
#include <govirt/ovirt-cluster.h>
#include <govirt/ovirt-collection.h>
#include <govirt/ovirt-data-center.h>
#include <govirt/ovirt-error.h>
#include <govirt/ovirt-host.h>
#include <govirt/ovirt-options.h>
#include <govirt/ovirt-proxy.h>
#include <govirt/ovirt-resource.h>
#include <govirt/ovirt-rest-call-error.h>
#include <govirt/ovirt-storage-domain.h>
#include <govirt/ovirt-vm.h>
#include <govirt/ovirt-vm-display.h>
#include <govirt/ovirt-vm-pool.h>

#endif /* __OVIRT_H__ */
